<?php

namespace domain\Producer\Producer;

use common\components\DolibarrApi;
use domain\Producer\ProducerPriceRange\ProducerPriceRangeRepository;
use domain\_\AbstractManager;
use yii\base\ErrorException;

class DolibarrProducerUtils extends AbstractManager
{
    protected DolibarrApi $dolibarrApi;
    protected ProducerPriceRangeRepository $producerPriceRangeRepository;
    protected ProducerRepository $producerRepository;

    public function loadDependencies(): void
    {
        $this->dolibarrApi = \Yii::$app->dolibarrApi;
        $this->producerPriceRangeRepository = $this->loadService(ProducerPriceRangeRepository::class);
        $this->producerRepository = $this->loadService(ProducerRepository::class);
    }

    public function getDolibarrProducerInvoices(Producer $producer): array
    {
        $invoicesArray = [];
        if($producer->dolibarr_socid) {
            $invoicesArray = $this->dolibarrApi->getInvoicesByThirParty($producer->dolibarr_socid);
        }

        return $invoicesArray;
    }

    public function countDolibarrProducerInvoicesUnpaid(Producer $producer): int
    {
        $count = 0;
        $invoicesArray = $this->getDolibarrProducerInvoices($producer);
        foreach($invoicesArray as $invoice) {
            if($invoice['remaintopay'] > 0) {
                $count ++;
            }
        }

        return $count;
    }

    public function generateDolibarrProducerInvoice(Producer $producer)
    {
        $idProduct = $this->getDolibarrProductId($producer);
        if(!$idProduct) {
            throw new ErrorException("Dolibarr : pas d'id product trouvé pour le producteur ".$producer->name);
        }

        $idInvoice = $this->dolibarrApi->createInvoice($producer->dolibarr_socid);
        $this->dolibarrApi->addInvoiceLine($idInvoice, $idProduct);
        $responseValidate = $this->dolibarrApi->validateInvoice($idInvoice);
        $this->dolibarrApi->generateInvoicePdf($responseValidate['ref']);
    }

    private function getDolibarrProductId(Producer $producer)
    {
        if($producer->dolibarr_product_id) {
            $idProduct = $producer->dolibarr_product_id;
        }
        else {
            $producerPriceRange = $this->producerPriceRangeRepository->getProducerPriceRangeByTurnover($this->producerRepository->getTurnoverLastMonth($producer));
            if($producerPriceRange && $producerPriceRange->dolibarr_product_id) {
                $idProduct = $producerPriceRange->dolibarr_product_id;
            }
        }

        return $idProduct;
    }
}